@@ -27,32 +27,32 @@ class AzureDatalakeFileSystem(AbstractFileSystem):
     Examples
     _________
     >>> adl = AzureDatalakeFileSystem(tenant_id="xxxx", client_id="xxxx", 
-                                    client_secret="xxxx", store_name="storage_account"
+                                    client_secret="xxxx"
                                     )
         adl.ls('')
         
         Sharded Parquet & csv files can be read as:
         ----------------------------
-        ddf = dd.read_parquet('adl://folder/filename.parquet', storage_options={
+        ddf = dd.read_parquet('adl://store_name/folder/filename.parquet', storage_options={
             'tenant_id': TENANT_ID, 'client_id': CLIENT_ID,
-            'client_secret': CLIENT_SECRET, 'store_name': STORE_NAME
+            'client_secret': CLIENT_SECRET
         })
 
-        ddf = dd.read_csv('adl://folder/*.csv', storage_options={
+        ddf = dd.read_csv('adl://store_name/folder/*.csv', storage_options={
             'tenant_id': TENANT_ID, 'client_id': CLIENT_ID,
-            'client_secret': CLIENT_SECRET, 'store_name': STORE_NAME
+            'client_secret': CLIENT_SECRET
         })
 
         Sharded Parquet and csv files can be written as:
         ------------------------------------------------
-        dd.to_parquet(ddf, 'adl://folder/filename.parquet, storage_options={
+        dd.to_parquet(ddf, 'adl://store_name/folder/filename.parquet, storage_options={
             'tenant_id': TENANT_ID, 'client_id': CLIENT_ID,
-            'client_secret': CLIENT_SECRET, 'store_name': STORE_NAME
+            'client_secret': CLIENT_SECRET
         })
         
-        ddf.to_csv('adl://folder/*.csv', storage_options={
+        ddf.to_csv('adl://store_name/folder/*.csv', storage_options={
             'tenant_id': TENANT_ID, 'client_id': CLIENT_ID,
-            'client_secret': CLIENT_SECRET, 'store_name': STORE_NAME
+            'client_secret': CLIENT_SECRET
         })
 
     Parameters
@@ -64,7 +64,8 @@ class AzureDatalakeFileSystem(AbstractFileSystem):
     client_secret: string
         The access key
     store_name: string (None)
-        The name of the datalake account being accessed
+        The name of the datalake account being accessed.  Should be inferred from the urlpath
+        if using with Dask read_xxx and to_xxx methods.
     """
 
     def __init__(self, tenant_id, client_id, client_secret, store_name):
@@ -75,6 +76,20 @@ class AzureDatalakeFileSystem(AbstractFileSystem):
         self.store_name = store_name
         self.do_connect()
 
+    @staticmethod
+    def _get_kwargs_from_urls(paths):
+        """ Get the store_name from the urlpath and pass to storage_options """
+        ops = infer_storage_options(paths)
+        out = {}
+        if ops.get('host', None):
+            out['store_name'] = ops['host']
+        return out
+
+    @classmethod
+    def _strip_protocol(cls, path):
+        ops = infer_storage_options(path)
+        return ops['path']
+    
     def do_connect(self):
         """Establish connection object."""
         token = lib.auth(tenant_id=self.tenant_id,
@@ -298,7 +313,8 @@ class AzureBlobFileSystem(AbstractFileSystem):
                             path_['size'] = int(path_.pop('contentLength'))
                         else: path_['size'] = int(0)
                     if len(pathlist) == 1:
-                        return pathlist[0]
+                        return pathlist
+                        # return pathlist[0]
                     else:
                         return pathlist
                 else:
@@ -350,7 +366,7 @@ class AzureBlobFile(AbstractBufferedFile):
                     cache_type=cache_type, autocommit=autocommit)
         self.fs = fs
         self.path = path
-
+        
     def _fetch_range(self, start=None, end=None):
         """ Gets the specified byte range from Azure Datalake Gen2 """
         if start is not None or end is not None:
@@ -385,60 +401,62 @@ class AzureBlobFile(AbstractBufferedFile):
         content_length = self.fs.info(path=self.path)['size']
         return content_length
 
-    # def _set_size(self, current_size, appended_size):
-    #     print('Setting size...')
-    #     headers = self.fs._make_headers(content_length=appended_size)
-    #     new_size = int(current_size) + int(appended_size)
-    #     headers['Content-Length'] = str(new_size)
-    #     url = self.fs._make_url(path=self.path)
-    #     params = {'action': 'setProperties'}
-    #     response = requests.patch(url=url, headers=headers,
-    #                               params=params)
-    #     if not response.status_code == requests.codes.ok:
-    #         response.raise_for_status()
-    #     print(f'New filesize is: {self._get_size()}')
-
-    def _upload_chunk(self, final: bool = False, resource: str = None):
-        """ Writes part of a multi-block file to Azure Datalake """
-
-        self.buffer.seek(0)
-        data = self.buffer.getvalue()
-        l = len(data)
-        
-        # Get the size of the existing file
-        current_size = self._get_size()
-        
-        # Append current buffer to the existing file
-        headers = self.fs._make_headers(content_length=l)
+    def _flush_file(self, position):
         url = self.fs._make_url(path=self.path)
-        # Set the parameters for the API query.  
-        # Can be one of "append", "flush".
-        # Other allowed values for PATCH on ADLGen2 are
-        # "setProperties" and "setAccessControl"
-        # For append, "position" must be the position where the data is to be appended
-        params = {'action': 'append',
-                  'position': 0}
-        response = requests.patch(url, headers=headers, data=data, params=params)
-        if not response.status_code == requests.codes.ok:
-            response.raise_for_status()
-        
-        # This is the flush operation
-        # To flush, the previously uploaded data must be contiguous, the position
-        # parameter must be specified, and equal to the length of the file after
-        # all data has been written, and there can be no request entity body
-        # in the request.
-        # To flush, must set header content-length == 0.
         params = {'action': 'flush',
-                  'position': l}
-        headers = self.fs._make_headers(media_type='application/octet-stream',
-                                        content_length=0)
+                  'position': position
+                  }
+        headers = self.fs._make_headers(media_type='text/plain',
+                                            content_length=0
+                                            )
         response = requests.patch(url, headers=headers, params=params)
         if not response.status_code == requests.codes.ok:
             response.raise_for_status()
+        return response.status_code
+        
+    def _append_file(self, position, content_length, data):
+        url = self.fs._make_url(path=self.path)
+        headers = self.fs._make_headers(media_type='text/plain',
+                                   content_length=content_length,
+                                   )
+        params = {'action': 'append',
+                'position': self.offset
+                }
+        response = requests.patch(url, headers=headers, params=params,
+                                  data=data)
+        if not response.status_code == requests.codes.ok:
+            response.raise_for_status()
+        return response.status_code
+
+    def _upload_chunk(self, final: bool = False, resource: str = None):
+        """ Writes part of a multi-block file to Azure Datalake """
+        self.buffer.seek(0)
+        data = self.buffer.getvalue()
+        l = len(data)
+        end_position = self.offset + l
+        
+        if final and l == 0:
+            status_code = self._flush_file(position=end_position)
+            if status_code!=202:
+                raise Exception(f'Flush to Azure Datalake failed with status code={status_code} on file {self.path}')
+        
+        elif final and l > 0:
+        # Append current buffer to the existing file
+            status_code = self._append_file(position=self.offset, content_length=l, data=data)
+            if status_code==202:
+                status_code = self._flush_file(position=end_position)
+                if status_code != 200:
+                    raise Exception(f'Flush to Azure Datalake failed with status code={status_code} on file {self.path}')
+                
+        elif not final and l > 0:
+            status_code = self._append_file(position=self.offset, content_length=l, data=data)
+            if status_code != 202:
+                raise Exception(f'Write to Azure Datalake failed with status code={status_code} on file {self.path}')
+                
+        else:
+            raise Exception(f'Unexpected condition during _upload_chunk for l={l}, final={final}, end_position={end_position}, path={self.path}')
             
-        # Then set the new file Content-Length on ADLS Gen2
-        # self._set_size(current_size=current_size, appended_size=l)
-            
+
     def upload_single_shot(self, final: bool = False):
         """ Writes an entire file to Azure Datalake """
         headers = self.fs._make_headers()
